package com.example.damien.whichfingers;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.support.annotation.NonNull;
import android.view.View;

public class DrawingView extends View implements  Comparable<DrawingView> {

    private Paint _paint = new Paint(Paint.ANTI_ALIAS_FLAG);
    private int _x = 0;
    private int _y = 0;
    private int _fid = -1;
    private boolean _sym = false;
    private boolean isDown = false;

    public void setFid(int fid) { _fid = fid; }

    public int getFid() {return _fid;}

    public void setSym(boolean val) { _sym = val; }

    public DrawingView(Context context) {
        super(context);
    }

    public void setPosition(int x, int y) {
        _x = x;
        _y = y;
    }

    public int getTouchX() {return _x;}
    public int getTouchY() {return _y;}

    public void reset() {
        _x = 0;
        _fid = -1;
        _sym = false;
        isDown = false;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        //QColor(163, 169, 72), QColor(237, 185, 46), QColor(248, 89, 49), QColor(206, 24, 54), QColor(0, 153, 137)

        _paint.setTextSize(60);
        _paint.setStyle(Style.FILL);

        int offset_x = 110;
        int offset_y = 160;
        canvas.save();
        if (_sym) {
            canvas.rotate(180, _x, _y);
            //offset_x = -offset_x;
            offset_y = -(offset_y+30);
        }
        switch (_fid) {
            case 0:
                canvas.drawText("Thumb", _x-offset_x, _y-offset_y, _paint);
                // Inner
                _paint.setARGB(255, 163, 169, 72);
                break;
            case 1:
                canvas.drawText("Index", _x-offset_x, _y-offset_y, _paint);
                // Inner
                _paint.setARGB(255, 237, 185, 46);
                break;
            case 2:
                canvas.drawText("Middle", _x-offset_x, _y-offset_y, _paint);
                // Inner
                _paint.setARGB(255, 248, 89, 49);
                break;
            case 3:
                canvas.drawText("Ring", _x-offset_x, _y-offset_y, _paint);
                // Inner
                _paint.setARGB(255, 206, 24, 54);
                break;
            case 4:
                canvas.drawText("Little", _x-offset_x, _y-offset_y, _paint);
                // Inner
                _paint.setARGB(255, 0, 153, 137);
                break;
            default:
                //canvas.drawText("Unknown", _x-offset_x, _y-offset_y, _paint);
                // Inner
                //_paint.setARGB(255, 153, 52, 4);
                _paint.setARGB(0, 0, 0, 0);
                break;
        }
        canvas.restore();

        if (_fid != 5) {
            _paint.setStyle(Style.FILL);
            canvas.drawCircle(_x, _y, 130, _paint);

            // Outter
            _paint.setARGB(255, 0, 0, 0);
            _paint.setStyle(Style.STROKE);
            _paint.setStrokeWidth(4);
            canvas.drawCircle(_x, _y, 130, _paint);
        }

        // Redraw the canvas
        invalidate();
    }

    public void setDown(boolean down) {
        isDown = down;
    }

    public boolean isDown() {
        return isDown;
    }

    @Override
    public int compareTo(@NonNull DrawingView o) {
        return _x - o._x;
    }
}