package com.example.damien.whichfingers;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Created by damien on 21/03/17.
 */

public class FingerTouchAssociationTask extends TimerTask {
    private final int DELAY = 30;
    private ArrayList<Touch> pendingTouches = new ArrayList<>();
    private int[] pendingPiezoValues = new int[5];
    private Timer timer;
    private DrawingView[] tab;
    private long treatPendingTouchesTimestamp = -1;
    private WhichFingersHID wf;
    private boolean[] fingerDown = new boolean[5];

    public FingerTouchAssociationTask(WhichFingersHID wf, DrawingView[] tab) {
        this.wf = wf;
        this.tab = tab;
        timer = new Timer();
        timer.schedule(this, 0, 2);
    }

    public void reset() {
        pendingTouches.clear();
        pendingPiezoValues = new int[5];
        treatPendingTouchesTimestamp = -1;
        fingerDown = new boolean[5];
    }

    synchronized public void addTouch(Touch touch) {
        touch.setPiezoIdx(wf.getCurrentIdx());
        pendingTouches.add(touch);
        // Wait 30ms before doing the association with a finger
        treatPendingTouchesTimestamp = System.currentTimeMillis() + DELAY;
    }

    synchronized public void removeTouch(Integer id) {
        int fingerId = tab[id].getFid();
        if (fingerId >= 0 && fingerId < fingerDown.length) {
            fingerDown[fingerId] = false;
        }


        //wf.logger.println("Remove touch: " + id + " finger id: " + fingerId + " pending touches: " + pendingTouches.size() + " ");

        Iterator<Touch> it = pendingTouches.iterator();
        while (it.hasNext()) {
            Touch t  = it.next();
            if (t.getId() == id) {
                it.remove();
            }
        }
    }

    synchronized public void addPiezoValues(byte[] values) {
        for (int i = 0; i < pendingPiezoValues.length; ++i) {
            int value = values[i] & 0xFF; // Convert to unsigned byte

            if (value > pendingPiezoValues[i]) {
                pendingPiezoValues[i] = value;
            }
        }
    }

    @Override
    synchronized public void run() {

        if (treatPendingTouchesTimestamp != -1 && System.currentTimeMillis() >= treatPendingTouchesTimestamp) {
            int[] piezoMaxValues = new int[5];
            String str = "----- Treat ----------\n";
            for (Touch touch : pendingTouches) {
                str  += "Touch (" + touch.getX() + ", " + touch.getY() + ") with " + (pendingTouches.size() - 1) + " pending\n";
                for (int i = 0; i < 5; ++i) {
                    if (fingerDown[i]) {
                        str += "already down\n";
                        piezoMaxValues[i] = -1;
                        continue;
                    }
                    for (int p = -40; p < 17; ++p) {
                        int val = wf.getPiezoVal(i, touch.getPiezoIdx() + p) & 0xFF;
                        if (p == 0) {
                            str += "[" + val + "] ";
                        } else {
                            str += val + " ";
                        }
                        if (val > piezoMaxValues[i]) {
                            piezoMaxValues[i] = val;
                        }
                    }
                    str += "\n";
                }
            }

            ArrayList<Integer> fingersUsed = new ArrayList<>(pendingTouches.size());
            for (int i = 0; i < pendingTouches.size(); ++i) {
                int fingerId = 0;
                for (int j = 1; j < 5; ++j) {
                    if (piezoMaxValues[j] > piezoMaxValues[fingerId]) {
                        fingerId = j;
                    }
                }
                fingersUsed.add(fingerId);
                // Remove the piezo value so that it's not associated twice
                piezoMaxValues[fingerId] = -1;
            }

            Collections.sort(fingersUsed);
            Collections.sort(pendingTouches);

            for (int i = 0; i < fingersUsed.size(); ++i) {
                tab[pendingTouches.get(i).getId()].setFid(fingersUsed.get(i));
                fingerDown[fingersUsed.get(i)] = tab[pendingTouches.get(i).getId()].isDown();
            }

            pendingTouches.clear();
            Arrays.fill(pendingPiezoValues, 0);
            treatPendingTouchesTimestamp = -1;
        }
    }
}
